/*========================================================================
 * $RCSfile: PatternVector.java,v $
 *
 * $Author: liddle $
 * $Revision: 1.2 $
 * $Date: 1998/08/06 16:28:39 $
 *
 * $Log: PatternVector.java,v $
 * Revision 1.2  1998/08/06  16:28:39  liddle
 * Added macro processing code.
 *
 * Revision 1.1  1998/07/14  04:06:17  liddle
 * Initial revision
 *
 *------------------------------------------------------------------------
 * Pattern Editor
 * Data Extraction Research Group
 * School of Accountancy and Information Systems
 * Marriott School of Management
 * 540 TNRB
 * Brigham Young University
 * Provo, UT 84602-3068
 * liddle@byu.edu
 *
 * Copyright (C) 1998, Board of Trustees of Brigham Young University
 *
 * Pattern Editor software, both binary and source (hereafter, Software)
 * is copyrighted by The Board of Trustees of Brigham Young University
 * (BYU), and ownership remains with BYU.
 *
 * BYU grants you (hereafter, Licensee) a license to use the Software
 * for academic, research and internal business purposes only, without a
 * fee.  Licensee may distribute the binary and source code (if released)
 * to third parties provided that the copyright notice and this statement
 * appears on all copies and that no charge is associated with such
 * copies.
 *
 * Licensee may not make derivative works without first arranging an
 * additional license agreement with BYU.
 *
 * Any Licensee wishing to make commercial use of the Software should
 * contact BYU to negotiate an appropriate license for such
 * commercial use.  Commercial use includes (1) integration of all or
 * part of the source code into a product for sale or license by or on
 * behalf of Licensee to third parties, or (2) distribution of the binary
 * code or source code to third parties that need it to utilize a
 * commercial product sold or licensed by or on behalf of Licensee.
 *
 * BYU MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.  BYU SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE
 * USERS OF THIS SOFTWARE.
 *
 * By using or copying this Software, Licensee agrees to abide by the
 * copyright law and all other applicable laws of the United States
 * including, but not limited to, export control laws, and the terms
 * of this license.  BYU shall have the right to terminate this license
 * immediately by written notice upon Licensee's breach of, or
 * non-compliance with, any of its terms.  Licensee may be held legally
 * responsible for any copyright infringement that is caused or
 * encouraged by Licensee's failure to abide by the terms of this license.
 *
 * Comments and questions are welcome and can be sent to liddle@byu.edu.
 */
/*------------------------------------------------------------------------
 *			IMPORTS
 */
import java.util.*;
import java.util.zip.DataFormatException;

/*========================================================================
 * CLASS PatternVector -- Vector of Pattern objects
 */
public class PatternVector extends Vector {

/*------------------------------------------------------------------------
 *			CONSTANTS
 */
public static final String	SEPARATOR = " : ";

/*------------------------------------------------------------------------
 *			ATTRIBUTES
 */
private com.oroinc.text.regex.Perl5Compiler	compiler;

/*------------------------------------------------------------------------
 *			CONSTRUCTORS
 */
public PatternVector()
{
  super();
  compiler = new com.oroinc.text.regex.Perl5Compiler();

} /* PatternVector constructor */

public PatternVector(int ic)
{
  super(ic);
  compiler = new com.oroinc.text.regex.Perl5Compiler();

} /* PatternVector constructor */

public PatternVector(int ic, int ci)
{
  super(ic, ci);
  compiler = new com.oroinc.text.regex.Perl5Compiler();

} /* PatternVector constructor */

/*------------------------------------------------------------------------
 *			MUTATORS
 */
public void
Add_Pattern()
{
  addElement(new Pattern(this));

} /* Add_Pattern */

public void
Add_Pattern(String new_id, boolean case_flag, String new_pat)
{
  addElement(new Pattern(this, new_id, case_flag, new_pat));

} /* Add_Pattern */

public void
Add_Pattern(String pattern_text) throws DataFormatException
{
  addElement(new Pattern(this, pattern_text));

} /* Add_Pattern */

/*------------------------------------------------------------------------
 *			ACCESSORS
 */
public Pattern
Find(String find_ID)
{
  Pattern	pat;

  for (int i = this.size() - 1; i >= 0; i--) {
    pat = Get(i);
    if (pat.Get_ID().equals(find_ID))
      return (pat);
  }
  return (null);

} /* Find */

public Pattern
Get(int i)
{
  return ((Pattern)elementAt(i));

} /* Get */

/*------------------------------------------------------------------------
 *			FACILITATORS
 */
public void
Invalidate_Dependents(String ID)
{
  Pattern	pat;
  String	macro = "{" + ID + "}";

  for (int i = size() - 1; i >= 0; i--) {
    pat = Get(i);
    if (pat.Get_Regular_Expression().indexOf(macro) >= 0) {
      pat.Invalidate_Pattern();
      Invalidate_Dependents(pat.Get_ID());
    }
  }

} /* Invalidate_Dependents */

public void
Move_Pattern(int row_from, int row_to)
{
  Object	move;

  move = elementAt(row_from);
  removeElementAt(row_from);
  insertElementAt(move, row_to);

} /* Move_Pattern */

public String 
Substitute_Patterns(String line) 
{
  int           close_pos, 
                open_pos = line.indexOf("{"); 
  String        key, 
                left, 
                middle, 
                right; 
  Pattern	pat; 

  while (open_pos >= 0) {
    close_pos = line.indexOf("}", open_pos + 1); 

    if (close_pos < 0) 
      break; 

    key = line.substring(open_pos + 1, close_pos); 

    pat = Find(key); 
    if (pat != null) { 
      left = line.substring(0, open_pos); 
      middle = Substitute_Patterns(pat.Get_Regular_Expression()); 
      right = line.substring(close_pos + 1); 
      line = left + middle + right; 

      close_pos = left.length() + middle.length() - 1;
    } 

    open_pos = line.indexOf("{", close_pos + 1); 
  } 
  return (line); 

} /* Substitute_Patterns */ 

/*========================================================================
 * INNER CLASS Pattern -- a pattern object 
 *                           (ID, case flag, pattern string, match vector)
 */
public class Pattern implements Cloneable {

/*------------------------------------
 *	ATTRIBUTES
 */
private String	ID;		/* Unique ID or name of this pattern	*/
private Boolean	sensitive;	/* True if pattern is case sensitive	*/
private String	reg_exp;	/* Perl5 regular expression		*/
private com.oroinc.text.regex.Pattern	pattern;
				/* ORO compiled pattern			*/
private PatternVector	patterns;
private MatchVector     matches;

/*------------------------------------
 *	CONSTRUCTORS
 */
private
Pattern(PatternVector pv)
{
  this(pv, "", false, "");

} /* Pattern constructor */

private
Pattern(PatternVector pv, String new_id, boolean new_case, String new_pat)
{
  patterns = pv;
  ID = new_id;
  sensitive = new Boolean(new_case);
  reg_exp = new_pat;
  pattern = null;
  matches = null;

} /* Pattern constructor */

private
Pattern(PatternVector pv, String pat) throws DataFormatException
{
  patterns = pv;
  Parse_Pattern(pat);
  matches = null;

} /* Pattern constructor */

/*------------------------------------
 *	ACCESSORS
 */
public Boolean Get_Case_Sensitive()	{ return (sensitive); }
public String Get_ID()			{ return (ID); }
public String Get_Regular_Expression()	{ return (reg_exp); }
public boolean Is_Case_Sensitive()	{ return (sensitive.
						  booleanValue()); }

public MatchVector
Get_MatchVector()
{
  return (matches);

} /* Get_MatchVector */

public com.oroinc.text.regex.Pattern
Get_Raw_Pattern()
{
  return (pattern);

} /* Get_Raw_Pattern */

public com.oroinc.text.regex.Pattern
Get_Pattern() throws com.oroinc.text.regex.MalformedPatternException
{
  if (pattern == null)
    Compile_Pattern();
  return (pattern);

} /* Get_Pattern */

/*------------------------------------
 *	FACILITATORS
 */
public Object
clone()
{
  return (new Pattern(patterns, ID, sensitive.booleanValue(), reg_exp));

} /* clone */

public void
Compile_Pattern() throws com.oroinc.text.regex.MalformedPatternException
{
  pattern = null;
  if (reg_exp != null && !reg_exp.equals("")) {
    String	subst_exp = patterns.Substitute_Patterns(reg_exp);

    // System.out.println("Compiling " + subst_exp);
    if (sensitive.booleanValue())
      pattern = compiler.compile(subst_exp);
    else
      pattern = compiler.compile(subst_exp,
	   com.oroinc.text.regex.Perl5Compiler.CASE_INSENSITIVE_MASK);
  }

} /* Compile_Pattern */

private void
Parse_Pattern(String str) throws DataFormatException
{
  int		pos;
  String	remainder;

  pos = str.indexOf(SEPARATOR);
  if (pos > 0) {
    ID = str.substring(0, pos);
    remainder = str.substring(pos + 3);
    pos = remainder.indexOf(SEPARATOR);
    if (pos > 0) {
      sensitive = Boolean.valueOf(remainder.substring(0, pos));
      reg_exp = remainder.substring(pos + 3);
    } else {
      throw new DataFormatException("Unable to parse pattern.");
    }
  } else {
    throw new DataFormatException("Unable to parse pattern.");
  }
  pattern = null;

} /* Parse_Pattern */

public String
toString()
{
  return (ID + SEPARATOR + sensitive + SEPARATOR + reg_exp);

} /* toString */

/*------------------------------------
 *	MUTATORS
 */
public void
Set_Case_Sensitivity(Boolean case_flag)
{
  if (!sensitive.equals(case_flag)) {
    sensitive = case_flag;
    pattern = null;
  }

} /* Set_Case_Sensitivity */

public void
Set_Case_Sensitivity(boolean case_flag)
{
  if (sensitive.booleanValue() != case_flag) {
    sensitive = new Boolean(case_flag);
    pattern = null;
  }

} /* Set_Case_Sensitivity */

public void
Set_ID(String new_id)
{
  // Invalidate anyone who used to depend on this.
  patterns.Invalidate_Dependents(ID);
  ID = new_id;
  // Invalidate anyone who now depends on this.
  patterns.Invalidate_Dependents(ID);

} /* Set_ID */

public void
Set_MatchVector(MatchVector mv)
{
  matches = mv;

} /* Set_MatchVector */

public void
Invalidate_Pattern()
{
  // Only called by PatternVector.
  pattern = null;

} /* Invalidate_Pattern */

public void
Set_Regular_Expression(String new_pat)
{
  if (!new_pat.equals(reg_exp)) {
    reg_exp = new_pat;
    pattern = null;
    patterns.Invalidate_Dependents(ID);
  }

} /* Set_Regular_Expression */

/*========================================================================
 * INNER CLASS MatchVector -- Vector of Match objects
 */
public class MatchVector extends Vector {

/*------------------------------------
 *	CONSTRUCTORS - MatchVector
 */
public MatchVector()
{ 
  super();

} /* MatchVector constructor */

public MatchVector(int ic)
{ 
  super(ic);  //sets the initial capacity for the vector;

} /* MatchVector constructor */

public MatchVector(int ic, int ci)
{ 
  super(ic, ci);  //sets initial capacity and capacity increment;

} /* MatchVector constructor */

/*------------------------------------
 *	MUTATORS - MatchVector
 */
public void
Add_Match(int start, int end)
{
  this.addElement(new Match(start, end));

} /* Add_Match */

/*------------------------------------
 *	ACCESSORS - MatchVector
 */
public Match
Get(int i)
{
  return ((Match)this.elementAt(i));

} /* Get */

/*========================================================================
 * INNER CLASS Match --  a Match Object (start, end positions)
 */
public class Match {

/*------------------------------------
 *	ATTRIBUTES - Match
 */
int	start_pos;
int	end_pos;

/*------------------------------------
 *	CONSTRUCTORS - Match
 */
public
Match(int start, int end) 
{
  start_pos = start;
  end_pos = end;

} /* Match constructor */

/*------------------------------------
 *	ACCESSORS - Match
 */
public int
Get_Start() 
{
  return (start_pos);

} /* Get_Start */

public int
Get_End() 
{
  return (end_pos);

} /* Get_End */

public String
toString()
{
  return ("Match (" + start_pos + ", " + end_pos + ")");

} /* toString */

/*------------------------------------
 *	MUTATORS - Match
 */
public void Set_Start(int startpos)
{
  start_pos = startpos;

} /* Set_Start */

public void Set_End(int endpos)
{
  end_pos = endpos;

} /* Set_End */

public void Set(int startpos, int endpos)
{
  start_pos = startpos;
  end_pos = endpos;

} /* Set */

} /* class Match */

} /* class MatchVector */

} /* class Pattern */

} /* class PatternVector */

/*========================================================================
 *			END OF FILE $RCSfile: PatternVector.java,v $
 */
